# orbbec_camera

---

ORBBEC_SDK ROS2 wrapper for Orbbec 3D Camera

## Installation Instructions

---

Install ROS2

* Please refer to the official [ROS 2 installation guide](https://docs.ros.org/en/galactic/Installation/Ubuntu-Install-Debians.html)

* Install additional dependencies
  
  ```bash
  sudo apt install libgflags-dev nlohmann-json3-dev \
  ros-galactic-image-transport ros-galactic-image-publisher
  ```

* Install glog
  
  ```bash
  wget -c https://github.com/google/glog/archive/refs/tags/v0.6.0.tar.gz  -O glog-0.6.0.tar.gz
  tar -xzvf glog-0.6.0.tar.gz
  cd glog-0.6.0
  mkdir build && cd build
  cmake .. && make -j4
  sudo make install
  sudo ldconfig
  ```

* Install `magic_enum`
  
  ```bash
  wget -c https://github.com/Neargye/magic_enum/archive/refs/tags/v0.8.0.tar.gz -O  magic_enum-0.8.0.tar.gz
  tar -xzvf magic_enum-0.8.0.tar.gz
  cd magic_enum-0.8.0
  mkdir build && cd build
  cmake .. && make -j4
  sudo make install
  sudo ldconfig
  ```
- Install `libusb.rules`
  
  ```
  cd orbbec_camera/scripts
  sudo bash install.sh
  sudo udevadm control --reload-rules && sudo udevadm trigger
  ```

Run with Orbbec Camera

* Create ros2 workspace
  
  ```bash
  mkdir -p ~/ros2_ws/src
  ```

* Copy Orbbec ROS2 SDK to ~/ros2_ws/src

* Compile
  
  ```bash
  cd ~/ros2_ws/src
  colcon build --event-handlers  console_direct+  --cmake-args  -DCMAKE_BUILD_TYPE=Release
  ```

---

## Getting started

---

Launch camera node

* On terminal 1
  
  ```bash
  . ./install/setup.bash 
  ros2 launch orbbec_camera ob_camera.launch.py
  ```

* On terminal 2
  
  ```bash
  . ./install/setup.bash 
  rviz2 -d src/orbbec_camera/rviz/pointcloud.rviz
  ```

* List topics / services/ parameters ( on terminal 3)
  
  ```bash
  ros2 topic list
  ros2 service list
  ros2 param list
  ```

* Show depth to color extrinsic
  
  ```bash
  ros2 topic echo --qos-durability=transient_local /camera/extrinsic/depth_to_color  --qos-profile=services_default
  ```

* Get camera info, check [ROS camera info](http://docs.ros.org/en/melodic/api/sensor_msgs/html/msg/CameraInfo.html) as reference
  
  ```bash
  ros2 topic echo /camera/depth/camera_info
  ros2 topic echo /camera/color/camera_info
  ```

* Get device info
  
  ```bash
  ros2 service call /camera/get_device_info orbbec_camera_msgs/srv/GetDeviceInfo '{}'
  ```

* Get SDK version
  
  ```bash
  ros2 service call /camera/get_sdk_version orbbec_camera_msgs/srv/GetString '{}'
  ```

* Get exposure
  
  ```bash
  ros2 service call /camera/get_color_exposure orbbec_camera_msgs/srv/GetInt32 '{}'
  ```

> For checking `ir` or `depth`, please change ` /camera/get_color_exposure`
> to  `/camera/get_ir_exposure` or `/camera/get_depth_exposure`, Same below.

* Get gain
  
  ```bash
  ros2 service call /camera/get_color_gain orbbec_camera_msgs/srv/GetInt32 '{}'
  ```

* Get white balance
  
  ```bash
  ros2 service call /camera/get_white_balance orbbec_camera_msgs/srv/GetInt32 '{}'
  ```

* Set auto exposure
  
  ```bash
  ros2 service call /camera/set_color_auto_exposure std_srvs/srv/SetBool '{data: false}'
  ```

* Set white balance
  
  ```bash
  ros2 service call /camera/set_white_balance orbbec_camera_msgs/srv/SetInt32 '{data: 4600}'
  ```

* Set laser enable
  
  ```bash
  ros2 service call  /camera/set_laser_enable std_srvs/srv/SetBool "{data: true}" 
  ```

* toggle sensor
  
  ```bash
  ros2 service call  /camera/toggle_ir std_srvs/srv/SetBool "{data : true}"
  ```

---

DDS Tuning

---

The default DDS settings may not be efficient for data transmission. Different DDS settings will have different performance. Here we take CycloneDDS as an example. For detailed information, please refer to [ROS DDS Tuning](https://docs.ros.org/en/humble/How-To-Guides/DDS-tuning.html)

* Configure cyclonedds
  
  ```bash
  sudo gedit /etc/cyclonedds/config.xml
  ```
  
  Add
  
  ```bash
  <?xml version="1.0" encoding="UTF-8"?>
  <CycloneDDS xmlns="https://cdds.io/config" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xsi:schemaLocation="https://cdds.io/config
  https://raw.githubusercontent.com/eclipse-cyclonedds/cyclonedds/master/etc/cyclonedds.xsd">
      <Domain id="any">
          <General>
              <NetworkInterfaceAddress>lo</NetworkInterfaceAddress>
              <AllowMulticast>false</AllowMulticast>
          </General>
          <Internal>
              <MinimumSocketReceiveBufferSize>16MB</MinimumSocketReceiveBufferSize>
          </Internal>
          <Discovery>
              <ParticipantIndex>auto</ParticipantIndex>
              <MaxAutoParticipantIndex>
                  30</MaxAutoParticipantIndex>
              <Peers>
                  <Peer address="localhost" />
              </Peers>
          </Discovery>
      </Domain>
  </CycloneDDS>
  ```

* Set environment variables, add in .zshrc or .bashrc
  
  ```bash
  export ROS_DOMAIN_ID=42 # 0 to 232
  export ROS_LOCALHOST_ONLY=1
  export CYCLONEDDS_URI=file:///etc/cyclonedds/config.xml
  ```
  
  Tip: to understand why the maximum ROS_DOMAIN_ID is 232, please visit [The ROS_DOMAIN_ID](https://docs.ros.org/en/humble/Concepts/About-Domain-ID.html)

* Increase receive buffer size
  
  Locate and config
  
  ```bash
  /etc/sysctl.d/10-cyclone-max.conf
  ```
  
  Add
  
  ```bash
  net.core.rmem_max=2147483647
  net.core.rmem_default=2147483647
  ```

---

## FAQ

---

* No point clouds or images on rviz2
  Default Relaibility Policy = Best Effort, please make sure rviz2's reliability policy is set to best effort. Or use ros2 topic info -v /xxx/topic to check if the setting for QoS is correct